-- SPDX-License-Identifier: CC0-1.0

let Genode = env:DHALL_GENODE

let Child = Genode.Init.Child

in  { config =
        Genode.Init::{
        , children =
            toMap
              { noux =
                  Child.flat
                    Child.Attributes::{
                    , binary = "noux"
                    , exitPropagate = True
                    , resources =
                        Genode.Init.Resources::{
                        , caps = 500
                        , ram = Genode.units.MiB 10
                        }
                    , routes = [ Genode.Init.ServiceRoute.parent "Timer" ]
                    , config =
                        Genode.Init.Config::{
                        , attributes =
                            toMap
                              { stdin = "/script"
                              , stdout = "/dev/log"
                              , stderr = "/dev/log"
                              }
                        , content =
                            [ Genode.Prelude.XML.text
                                ''
                                <fstab>
                                	<tar name="bash-minimal.tar" />
                                	<dir name="dev"> <log/> <null/> <zero/> </dir>
                                	<dir name="tmp"> <ram /> </dir>
                                <inline name="script">
                                echo "hello world"
                                </inline>
                                </fstab>
                                <start name="/bin/bash">
                                	<env name="TERM" value="screen" />
                                </start>
                                ''
                            ]
                        }
                    }
              }
        }
    , rom =
        let manifest = env:MANIFEST
        
        in  Genode.Boot.toRomPaths
              [ manifest.bash-minimal.tar.bash-minimal
              , manifest.libc.lib.libc
              , manifest.libc.lib.libm
              , manifest.ncurses.lib.ncurses
              , manifest.noux.bin.noux
              , manifest.noux.lib.libc_noux
              , manifest.os.lib.vfs
              , manifest.posix.lib.posix
              ]
    }
