-- SPDX-License-Identifier: CC0-1.0

let Genode = env:DHALL_GENODE

let Init = Genode.Init

let Child = Init.Child

in  { config =
        Init::{
        , children =
            toMap
              { solo5 =
                  Child.flat
                    Child.Attributes::{
                    , binary = "solo5-test_blk"
                    , exitPropagate = True
                    , resources =
                        Init.Resources::{ caps = 256, ram = Genode.units.MiB 3 }
                    , routes =
                        [ Init.ServiceRoute.parent "Timer"
                        , Init.ServiceRoute.child "Block" "block"
                        ]
                    }
              , block =
                  Child.flat
                    Child.Attributes::{
                    , binary = "ram_block"
                    , provides = [ "Block" ]
                    , resources = Init.Resources::{ ram = Genode.units.MiB 9 }
                    , config =
                        Init.Config::{
                        , attributes =
                            toMap { size = "8M", block_size = "4096" }
                        }
                    }
              }
        }
    , rom =
        let manifest = env:MANIFEST
        
        in  Genode.BootModules.toRomPaths
              [ manifest.solo5-tests.bin.solo5-test_blk
              , manifest.os.bin.ram_block
              ]
    }
